import { doc } from '../../types/Window';
import { getCursorSegment, getInputValue, getLabelTextOf } from '../get/get';
import { a11yID } from '../staticValues';
export var a11yCreate = function (document) {
    var _a;
    if (document === void 0) { document = doc; }
    if (!document) {
        return null;
    }
    var $block = document.createElement('div');
    $block.setAttribute('aria-live', 'polite');
    $block.setAttribute('style', 'position: absolute; opacity: 0; height: 0; width: 0; overflow: hidden; pointer-events: none;');
    $block.id = a11yID;
    (_a = document.querySelector('body')) === null || _a === void 0 ? void 0 : _a.appendChild($block);
    return $block;
};
export var a11yUpdate = function ($input, announcementArray, document) {
    if (document === void 0) { document = doc; }
    if (!$input || !document)
        return '';
    a11yClear(document);
    var cursorSegment = getCursorSegment($input);
    var values = getInputValue($input).asTimeObject();
    var value = values[cursorSegment];
    var segmentValue = value === null ? 'blank' : value;
    var segmentName = {
        hrs12: 'Hours',
        minutes: 'Minutes',
        mode: 'AM/PM',
    }[cursorSegment];
    var announcements = {
        initial: '$label grouping $fullValue.',
        select: '$segmentName spin button $segmentValue.',
        update: '$segmentValue.',
    };
    var textArray = announcementArray.map(function (key) { return announcements[key]; });
    var fullValue = $input.value.replace(/--/g, 'blank');
    var html = "<p>" + textArray.join('</p><p>') + "</p>";
    var labelText = getLabelTextOf($input, document);
    html = html.replace(/\$label/g, labelText);
    html = html.replace(/\$segmentName/g, segmentName);
    html = html.replace(/\$segmentValue/g, "" + segmentValue);
    html = html.replace(/\$fullValue/g, fullValue);
    var $a11y = document.getElementById(a11yID);
    if ($a11y) {
        $a11y.innerHTML = html;
    }
    return html;
};
export var a11yClear = function (document) {
    if (document === void 0) { document = doc; }
    var $a11y = document === null || document === void 0 ? void 0 : document.getElementById(a11yID);
    if ($a11y) {
        $a11y.innerHTML = '';
    }
};
export var getA11yValue = function (document) {
    if (document === void 0) { document = doc; }
    var $a11y = document === null || document === void 0 ? void 0 : document.getElementById(a11yID);
    return ($a11y === null || $a11y === void 0 ? void 0 : $a11y.textContent) ? $a11y.textContent : '';
};
export var getA11yElement = function (document) {
    if (document === void 0) { document = doc; }
    return document === null || document === void 0 ? void 0 : document.getElementById(a11yID);
};
