import { blankValues } from '../../common/index';
import { isAmHrs24, isAmString24hr, isPmString24hr } from '../is/is';
import { regex } from '../regex/regex';
import { toLeadingZero, toNumber } from '../utils/utils';
import { validateHours24, validateString12hr, validateString24hr, validateTimeObject, } from '../validate/validate';
/** Utility for converting a 12 hour time string into either a 24 hour string or a time object. */
export var convertString12hr = function (string12hr) {
    validateString12hr(string12hr);
    return {
        to24hr: function () {
            if (/-/.test(string12hr))
                return '';
            var timeObject = convertString12hr(string12hr).toTimeObject();
            return convertTimeObject(timeObject).to24hr();
        },
        toTimeObject: function () {
            var result = regex.string12hr.exec(string12hr) || [];
            var _a = [
                toNumber(result[1]),
                toNumber(result[2]),
                (result[3] === '--' ? null : result[3]),
            ], hrs12 = _a[0], minutes = _a[1], mode = _a[2];
            var getHrs24 = function () {
                if (typeof hrs12 === 'number') {
                    if (mode === null && hrs12 === 12) {
                        return 0;
                    }
                    if (mode === 'PM') {
                        if (hrs12 === 12) {
                            return 12;
                        }
                        else if (hrs12 + 12 > 23) {
                            return (hrs12 + 12 - 24);
                        }
                        else {
                            return (hrs12 + 12);
                        }
                    }
                    else if (mode === 'AM' && hrs12 === 12) {
                        return 0;
                    }
                    else {
                        return hrs12;
                    }
                }
                return null;
            };
            var nullifyDashes = function (value) {
                return value === '--' ? null : value;
            };
            var newMode = nullifyDashes(mode);
            var hrs24 = newMode ? getHrs24() : null;
            var timeObject = {
                hrs24: hrs24,
                hrs12: nullifyDashes(hrs12),
                minutes: nullifyDashes(minutes),
                mode: newMode,
            };
            validateTimeObject(timeObject);
            return timeObject;
        },
    };
};
/** Utility for converting a 24 hour time string into either a 12 hour string or a time object. */
export var convertString24hr = function (string24hr) {
    validateString24hr(string24hr);
    return {
        to12hr: function () {
            if (string24hr === blankValues.string24hr) {
                return blankValues.string12hr;
            }
            var timeObject = convertString24hr(string24hr).toTimeObject();
            return convertTimeObject(timeObject).to12hr();
        },
        toTimeObject: function () {
            if (string24hr === blankValues.string24hr) {
                return blankValues.timeObject;
            }
            // string24hr
            var regResult = regex.string24hr.exec(string24hr) || [];
            var _a = [regResult[1], regResult[2]], hrsString24 = _a[0], minString = _a[1];
            var _b = [toNumber(hrsString24), toNumber(minString)], hrs24 = _b[0], minutes = _b[1];
            var timeObject = {
                hrs24: hrs24,
                hrs12: convertHours24(hrs24).toHours12(),
                minutes: minutes,
                mode: (isAmString24hr(string24hr) && 'AM') ||
                    (isPmString24hr(string24hr) && 'PM') ||
                    null,
            };
            validateTimeObject(timeObject);
            return timeObject;
        },
    };
};
/** Utility for converting a time object into either a 12 hour string or a 24 hour string. */
export var convertTimeObject = function (timeObject, skipValidation) {
    if (skipValidation === void 0) { skipValidation = false; }
    if (!skipValidation) {
        validateTimeObject(timeObject);
    }
    var hrs24 = timeObject.hrs24, hrs12 = timeObject.hrs12, minutes = timeObject.minutes, mode = timeObject.mode;
    var hrsString24 = toLeadingZero(hrs24);
    var hrsString12 = toLeadingZero(hrs12);
    var minString = toLeadingZero(minutes);
    return {
        to12hr: function () { return hrsString12 + ":" + minString + " " + (mode || '--'); },
        to24hr: function () {
            var string24hr = hrsString24 + ":" + minString;
            if (/-/.test(string24hr))
                return '';
            return string24hr;
        },
    };
};
/** Utility for converting a 24hr hours number into a 12hr hours number. */
export var convertHours24 = function (hours24) {
    validateHours24(hours24);
    var getHours12 = function () {
        if (typeof hours24 === 'number') {
            if (hours24 <= 12) {
                if (hours24 === 0) {
                    return 12;
                }
                else {
                    return hours24;
                }
            }
            else {
                return (hours24 - 12);
            }
        }
        return hours24;
    };
    return {
        toHours12: function () { return getHours12(); },
    };
};
/** Utility for converting a date object into either a 12hr string, a 24hr string or a time object. */
export var convertDateObject = function (date) {
    return {
        to12hr: function () {
            var timeObject = convertDateObject(date).toTimeObject();
            return convertTimeObject(timeObject).to12hr();
        },
        to24hr: function () {
            var timeObject = convertDateObject(date).toTimeObject();
            return convertTimeObject(timeObject).to24hr();
        },
        toTimeObject: function () {
            var _a = [date.getHours(), date.getMinutes()], hrs24 = _a[0], minutes = _a[1];
            var hrs12 = convertHours24(hrs24).toHours12();
            var mode = isAmHrs24(hrs24) ? 'AM' : 'PM';
            return { hrs24: hrs24, hrs12: hrs12, minutes: minutes, mode: mode };
        },
    };
};
