var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { doc } from '../../types/Window';
import { convertString12hr, convertString24hr } from '../convert/convert';
import { isString12hr, isString24hr } from '../is/is';
import { regex } from '../regex/regex';
import { ranges, rangesList } from '../staticValues';
var traverseSegmentRanges = function ($input, direction) {
    var cursorSegmentRange = getRangeOf($input).cursorSegment();
    var currentType = cursorSegmentRange.segment;
    var modifier = direction === 'forward' ? 1 : -1;
    var nextTypeIndex = rangesList.map(function (range) { return range.segment; }).indexOf(currentType) + modifier;
    return rangesList[nextTypeIndex] || cursorSegmentRange;
};
/** Essentiallly an alias for `convertString12hr(string12hr).toTimeObject()` */
export var getString12hr = function (string12hr) {
    var timeObject = convertString12hr(string12hr).toTimeObject();
    return __assign(__assign({}, timeObject), { timeObject: timeObject });
};
/** Essentiallly an alias for `convertString24hr(string24hr).toTimeObject()` */
export var getString24hr = function (string24hr) {
    var timeObject = convertString24hr(string24hr).toTimeObject();
    return __assign(__assign({}, timeObject), { timeObject: timeObject });
};
/** If you don't know if a string is 24hr or 12hr format, pass it into this function and it will consistently return in the desired time format that you want. */
export var provideTimeString = function (timeString) {
    var is12hr = isString12hr(timeString);
    var is24hr = isString24hr(timeString);
    if (!is12hr && !is24hr) {
        throw new Error("\"" + timeString + "\" is not a valid time string. Must be either in either 12 or 24 hour time format.");
    }
    return {
        as24hr: function () {
            if (is12hr) {
                return convertString12hr(timeString).to24hr();
            }
            return timeString;
        },
        as12hr: function () {
            if (is24hr) {
                return convertString24hr(timeString).to12hr();
            }
            return timeString;
        },
    };
};
/** Retrieve the current input value as either a 12hr string, a 24hr string or a time object. */
export var getInputValue = function ($input) {
    var value = ($input === null || $input === void 0 ? void 0 : $input.value) || '';
    var is12hrTime = regex.string12hr.test(value);
    var is24hrTime = regex.string24hr.test(value);
    return {
        as12hrString: function () { return (is12hrTime ? value : convertString24hr(value).to12hr()); },
        as24hrString: function () { return (is24hrTime ? value : convertString12hr(value).to24hr()); },
        asTimeObject: function () {
            return is12hrTime
                ? convertString12hr(value).toTimeObject()
                : convertString24hr(value).toTimeObject();
        },
    };
};
export var getLabelTextOf = function ($input, document) {
    if (document === void 0) { document = doc; }
    if (!$input)
        return '';
    var labelText = aria_labelledby($input, document) ||
        aria_label($input) ||
        for_attribute($input, document) ||
        label_wrapper_element($input) ||
        title_attribute($input);
    if (labelText)
        return labelText;
    console.error('Label text for input not found.', $input);
    throw new Error('Cannot polyfill time input due to a missing label.');
};
/** Retrieve the currently selected segment. */
export var getCursorSegment = function ($input) {
    return getRangeOf($input).cursorSegment().segment;
};
/** Retrieve the segment before the selected segment. */
export var getPrevSegment = function ($inputOrSegment) {
    if (typeof $inputOrSegment === 'string') {
        if ($inputOrSegment === 'hrs12')
            return 'hrs12';
        if ($inputOrSegment === 'minutes')
            return 'hrs12';
        if ($inputOrSegment === 'mode')
            return 'minutes';
    }
    return getRangeOf($inputOrSegment).prevSegment().segment;
};
/** Retrieve the segment after the selected segment. */
export var getNextSegment = function ($inputOrSegment) {
    if (typeof $inputOrSegment === 'string') {
        if ($inputOrSegment === 'hrs12')
            return 'minutes';
        if ($inputOrSegment === 'minutes')
            return 'mode';
        if ($inputOrSegment === 'mode')
            return 'mode';
    }
    return getRangeOf($inputOrSegment).nextSegment().segment;
};
/** Retrieve the cursor ranges of various segments. Used for making selections. */
export var getRangeOf = function ($input) { return ({
    rawSelection: function () {
        if (!$input) {
            return {
                start: 0,
                end: 0,
                segment: 'hrs12',
            };
        }
        var within = function (segment, value) {
            return ranges[segment].start <= value && value <= ranges[segment].end;
        };
        var start = $input.selectionStart;
        var end = $input.selectionEnd;
        var segment = (within('mode', start) && 'mode') || (within('minutes', start) && 'minutes') || 'hrs12';
        return {
            start: start,
            end: end,
            segment: segment,
        };
    },
    cursorSegment: function () {
        var segment = getRangeOf($input).rawSelection().segment;
        return ranges[segment];
    },
    nextSegment: function () { return traverseSegmentRanges($input, 'forward'); },
    prevSegment: function () { return traverseSegmentRanges($input, 'backward'); },
}); };
/** Retrieve a list of ancestor elements for a specific element. */
export var getAncestorsOf = function ($startingElem, selectorString) {
    // https://stackoverflow.com/a/8729274/1611058
    var $elem = $startingElem;
    var ancestors = [];
    var i = 0;
    while ($elem) {
        if (i !== 0) {
            ancestors.push($elem);
        }
        if (selectorString) {
            var matchesSelector = $elem.msMatchesSelector
                ? $elem.msMatchesSelector(selectorString) // IE Hack
                : $elem.matches(selectorString);
            if (matchesSelector) {
                return ancestors;
            }
        }
        $elem = $elem === null || $elem === void 0 ? void 0 : $elem.parentElement;
        i++;
    }
    return ancestors;
};
var elemText = function ($elem) { var _a; return ((_a = $elem === null || $elem === void 0 ? void 0 : $elem.textContent) === null || _a === void 0 ? void 0 : _a.trim()) || ''; };
function aria_labelledby($input, document) {
    if (document === void 0) { document = doc; }
    var ariaLabelByID = $input === null || $input === void 0 ? void 0 : $input.getAttribute('aria-labelledby');
    if (ariaLabelByID && document) {
        var $ariaLabelBy = document.getElementById(ariaLabelByID);
        return elemText($ariaLabelBy);
    }
    return '';
}
function aria_label($input) {
    var ariaLabel = $input.getAttribute('aria-label');
    return ariaLabel || '';
}
function for_attribute($input, document) {
    if (document === void 0) { document = doc; }
    var $forLabel = (document === null || document === void 0 ? void 0 : document.querySelector('label[for="' + $input.id + '"]')) || null;
    return elemText($forLabel);
}
function label_wrapper_element($input) {
    var ancestors = getAncestorsOf($input, 'label');
    var $parentLabel = ancestors[ancestors.length - 1];
    if ($parentLabel.nodeName == 'LABEL')
        return elemText($parentLabel);
    return '';
}
function title_attribute($input) {
    var titleLabel = $input.getAttribute('title');
    return titleLabel || '';
}
