import { getKeys } from '../../helpers/utils';
import { win } from '../../types/Window';
import { regex } from '../regex/regex';
import { timeObjectKeys } from '../staticValues';
import { toNumber } from '../utils/utils';
/** Retrieve the current state of the `[shift]` key. (Warning: will fail if bubbling is prevented) */
export var isShiftHeldDown = false;
win === null || win === void 0 ? void 0 : win.addEventListener('keyup', function (e) { return (isShiftHeldDown = e.shiftKey); });
win === null || win === void 0 ? void 0 : win.addEventListener('keydown', function (e) { return (isShiftHeldDown = e.shiftKey); });
var isValidTimeString = function (_a) {
    var value = _a.value, format = _a.format, minHrs = _a.minHrs, maxHrs = _a.maxHrs;
    var isFormatValid = regex[format].test(value);
    if (!isFormatValid)
        return false;
    var parsedString = regex[format].exec(value) || [];
    var hrsVal = toNumber(parsedString[1]);
    var minsVal = toNumber(parsedString[2]);
    var isHrsValid = hrsVal === null || (hrsVal >= minHrs && hrsVal <= maxHrs);
    var isMinsValid = minsVal === null || (minsVal >= 0 && minsVal <= 59);
    return isHrsValid && isMinsValid;
};
/** Check if a 24hr hours value is a PM value. */
export var isPmHrs24 = function (hrs24) { return hrs24 !== null && 12 <= hrs24 && hrs24 < 24; };
/** Check if a 12hr string value is a PM value. */
export var isPmString12hr = function (string12hr) { var _a; return ((_a = regex.string12hr.exec(string12hr)) === null || _a === void 0 ? void 0 : _a[3]) === 'PM'; };
/** Check if a 24hr string value is a PM value. */
export var isPmString24hr = function (string24hr) {
    var _a;
    if (string24hr === '')
        return false;
    var hrs24 = toNumber(((_a = regex.string24hr.exec(string24hr)) === null || _a === void 0 ? void 0 : _a[1]) || '');
    return typeof hrs24 == 'number' && hrs24 > 11;
};
/** Check if a time object is a PM value. */
export var isPmTimeObject = function (timeObject) {
    if (!timeObject.mode && timeObject.hrs24 !== null) {
        return timeObject.hrs24 > 11;
    }
    return timeObject.mode === 'PM';
};
/** Check if a 24hr hours value is an AM value. */
export var isAmHrs24 = function (hrs24) { return hrs24 !== null && !isPmHrs24(hrs24); };
/** Check if a 12hr string value is a AM value. */
export var isAmString12hr = function (string12hr) {
    return regex.string12hr.test(string12hr) &&
        string12hr.indexOf('--') === -1 &&
        !isPmString12hr(string12hr);
};
/** Check if a 24hr string value is a AM value. */
export var isAmString24hr = function (string24hr) {
    return string24hr !== '' && !isPmString24hr(string24hr);
};
/** Check if a time object is a AM value. */
export var isAmTimeObject = function (timeObject) {
    return !isPmTimeObject(timeObject) && isCompleteTimeObject(timeObject);
};
/** Check if a value is a time object */
export var isTimeObject = function (value) {
    if (typeof value === 'undefined' || typeof value !== 'object')
        return false;
    var keys = Object.keys(value);
    if (keys.length === 0)
        return false;
    var filteredKeys = timeObjectKeys.filter(function (key) { return keys.indexOf(key) === -1; });
    var additionalKeys = keys.filter(function (key) {
        // key might not be a TimeObjectKey but that is exactly what I'm checking for here
        return timeObjectKeys.indexOf(key) === -1;
    });
    return filteredKeys.length === 0 && additionalKeys.length === 0;
};
/** Check if a time object has no null any values. */
export var isCompleteTimeObject = function (timeObject) {
    if (!isTimeObject(timeObject)) {
        return false;
    }
    return getKeys(timeObject).every(function (key) { return timeObject[key] !== null; });
};
/** Check if a value is in a 12hr string format. */
export var isString12hr = function (value) {
    return isValidTimeString({ value: value, format: 'string12hr', minHrs: 1, maxHrs: 12 });
};
/** Check if a value is in a 24hr string format. */
export var isString24hr = function (value) {
    if (value === '')
        return true;
    return isValidTimeString({ value: value, format: 'string24hr', minHrs: 0, maxHrs: 23 });
};
