import { doc } from '../../types/Window';
import { getRangeOf } from '../get/get';
import { ranges } from '../staticValues';
import { toArray } from '../utils/utils';
/** Essentially `document.querySelectAll()` but it returns an array of elements instead of a node list. */
export var selectAll = function (selector, startingElem) {
    if (startingElem === void 0) { startingElem = doc; }
    var elements = startingElem === null || startingElem === void 0 ? void 0 : startingElem.querySelectorAll(selector);
    return toArray(elements || []);
};
/** Select a specific segment of a time input polyfill. */
export var selectSegment = function ($input, segment) {
    if (segment === void 0) { segment = 'hrs12'; }
    if (!$input)
        return;
    $input.setSelectionRange(ranges[segment].start, ranges[segment].end);
};
/** Select the segment after the selected segment of a time input polyfill. */
export var selectNextSegment = function ($input) {
    if (!$input)
        return;
    var _a = getRangeOf($input).nextSegment(), start = _a.start, end = _a.end;
    $input.setSelectionRange(start, end);
};
/** Select the segment before the selected segment of a time input polyfill. */
export var selectPrevSegment = function ($input) {
    if (!$input)
        return;
    var _a = getRangeOf($input).prevSegment(), start = _a.start, end = _a.end;
    $input.setSelectionRange(start, end);
};
/** Select the segment that the cursor is currently sitting in. */
export var selectCursorSegment = function ($input) {
    if (!$input)
        return;
    var _a = getRangeOf($input).cursorSegment(), start = _a.start, end = _a.end;
    $input.setSelectionRange(start, end);
};
