import { Hour12, Minute, Mode, Segment, String12hr, TimeObject } from '../../types/index';
declare type BlankFunc = () => void;
declare type ZeroToNine = 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9;
declare type GenericEntry = ZeroToNine | 'a' | 'p' | string;
declare type GenericEntries = Array<GenericEntry>;
declare type OnUpdate = (wasLimitHit: boolean) => void;
interface SegmentLogConstructor {
    startingValue: Hour12 | Minute | Mode;
    segment: Segment;
    onUpdate: OnUpdate;
    onLimitHit: BlankFunc;
}
declare class SegmentLog {
    value: Hour12 | Minute | Mode;
    segment: Segment;
    entries: GenericEntries;
    update: OnUpdate;
    limitHit: () => void;
    constructor({ startingValue, segment, onUpdate, onLimitHit }: SegmentLogConstructor);
    /**
     * Adds a value to the to the log and keeps track of what the end value should be
     * @param keyName - Expected to be a keyboard key name like "1" or "a"
     */
    add(keyName: string): void;
    /**
     * Reset is needed for things like typing "1", then leaving, then coming back.
     *
     * The tracker should reset if they are returning.
     */
    reset(): void;
    /**
     * Deletes the current value. Use this if the user presses delete or backspace.
     */
    clear(): void;
}
interface ManualEntryLogConstructor {
    /** The current Time object value */
    timeObject: TimeObject;
    /** Callback function for when the values change */
    onUpdate?: (entryLog: ManualEntryLog, wasLimitHit: boolean) => void;
    /** Callback function for when the manual entry exceeds the maximum range */
    onLimitHit?: (entryLog: ManualEntryLog) => void;
}
/**
 * Used for keeping track of Manual key strokes inside a time input
 */
export declare class ManualEntryLog {
    hrs12: SegmentLog;
    minutes: SegmentLog;
    mode: SegmentLog;
    fullValue12hr: String12hr;
    constructor({ timeObject, onUpdate, onLimitHit }: ManualEntryLogConstructor);
    /**
     * Deletes all of the values for all of the segments.
     */
    clearAll(): void;
}
export {};
