(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../../types/Window", "../get/get", "../staticValues", "../utils/utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.selectCursorSegment = exports.selectPrevSegment = exports.selectNextSegment = exports.selectSegment = exports.selectAll = void 0;
    var Window_1 = require("../../types/Window");
    var get_1 = require("../get/get");
    var staticValues_1 = require("../staticValues");
    var utils_1 = require("../utils/utils");
    /** Essentially `document.querySelectAll()` but it returns an array of elements instead of a node list. */
    exports.selectAll = function (selector, startingElem) {
        if (startingElem === void 0) { startingElem = Window_1.doc; }
        var elements = startingElem === null || startingElem === void 0 ? void 0 : startingElem.querySelectorAll(selector);
        return utils_1.toArray(elements || []);
    };
    /** Select a specific segment of a time input polyfill. */
    exports.selectSegment = function ($input, segment) {
        if (segment === void 0) { segment = 'hrs12'; }
        if (!$input)
            return;
        $input.setSelectionRange(staticValues_1.ranges[segment].start, staticValues_1.ranges[segment].end);
    };
    /** Select the segment after the selected segment of a time input polyfill. */
    exports.selectNextSegment = function ($input) {
        if (!$input)
            return;
        var _a = get_1.getRangeOf($input).nextSegment(), start = _a.start, end = _a.end;
        $input.setSelectionRange(start, end);
    };
    /** Select the segment before the selected segment of a time input polyfill. */
    exports.selectPrevSegment = function ($input) {
        if (!$input)
            return;
        var _a = get_1.getRangeOf($input).prevSegment(), start = _a.start, end = _a.end;
        $input.setSelectionRange(start, end);
    };
    /** Select the segment that the cursor is currently sitting in. */
    exports.selectCursorSegment = function ($input) {
        if (!$input)
            return;
        var _a = get_1.getRangeOf($input).cursorSegment(), start = _a.start, end = _a.end;
        $input.setSelectionRange(start, end);
    };
});
